==> picolibc_1.8.9.orig.tar.gz: picolibc-1.8.9/semihost/gettimeofday.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2021 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/time.h>
#include "semihost.h"

int
gettimeofday(struct timeval *restrict tv, void *restrict tz)
{
    uint64_t ticks;
    uint64_t elapsed;
    static uint64_t start_elapsed;
    static uint64_t start_ticks;
    static uintptr_t tick_freq;
    static int been_here;

    (void) tz;
    elapsed = sys_semihost_elapsed();
    if (!been_here) {
        start_elapsed = elapsed;
        tick_freq = sys_semihost_tickfreq();
        start_ticks = (uint64_t) sys_semihost_time() * tick_freq;
        been_here = 1;
    }

    ticks = start_ticks + (elapsed - start_elapsed);

    tv->tv_sec = (time_t) (ticks / tick_freq);
    tv->tv_usec = (suseconds_t) (ticks % tick_freq) * 1000000 / tick_freq;
    return 0;
}

==> picolibc_1.8.10.orig.tar.gz: picolibc-1.8.10/semihost/gettimeofday.c <==
/*
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Copyright © 2021 Keith Packard
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/time.h>
#include "semihost.h"

int
gettimeofday(struct timeval *restrict tv, void *restrict tz)
{
    uint64_t ticks;
    uint64_t elapsed;
    static uint64_t start_elapsed;
    static uint64_t start_ticks;
    static uintptr_t tick_freq;
    static int been_here;

    (void) tz;
    elapsed = sys_semihost_elapsed();
    if (!been_here) {
        start_elapsed = elapsed;
        tick_freq = sys_semihost_tickfreq();
        start_ticks = (uint64_t) sys_semihost_time() * tick_freq;
        been_here = 1;
    }

    ticks = start_ticks + (elapsed - start_elapsed);

    tv->tv_sec = (time_t) (ticks / tick_freq);
    tv->tv_usec = (suseconds_t) ((ticks % tick_freq) * 1000000 / tick_freq);
    return 0;
}
